#!/usr/bin/env python
# -*- coding: utf-8 -*-

# UVSnap Module.

"""
UVSnap Module.
~~~~


:author: Greg Albrecht <oss@undef.net>
:copyright: Copyright 2017 Greg Albrecht
:license: Apache License, Version 2.0
:source: <https://github.com/ampledata/uvsnap>

"""

from .constants import LOG_FORMAT, LOG_LEVEL  # NOQA
from .classes import NVR  # NOQA

__author__ = 'Greg Albrecht <oss@undef.net>'
__copyright__ = 'Copyright 2017 Greg Albrecht'
__license__ = 'Apache License, Version 2.0'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""UVSnap Module Class Definitions."""

import json
import logging
import logging.handlers

import requests

import uvsnap

__author__ = 'Greg Albrecht <oss@undef.net>'
__copyright__ = 'Copyright 2017 Greg Albrecht'
__license__ = 'Apache License, Version 2.0'


requests.packages.urllib3.disable_warnings()


class NVR(object):

    """NVR Object."""

    _logger = logging.getLogger(__name__)
    if not _logger.handlers:
        _logger.setLevel(uvsnap.LOG_LEVEL)
        _console_handler = logging.StreamHandler()
        _console_handler.setLevel(uvsnap.LOG_LEVEL)
        _console_handler.setFormatter(uvsnap.LOG_FORMAT)
        _logger.addHandler(_console_handler)
        _logger.propagate = False

    def __init__(self, nvr_url, api_key):
        self.nvr_url = nvr_url
        self.api_key = api_key

        self.cameras = []

    def get_cameras(self):
        """
        Gets a cameras from the NVR.
        """
        cameras_url = "%s/api/2.0/camera" % self.nvr_url

        get_cameras = requests.get(
            cameras_url,
            params={'apiKey': self.api_key},
            verify=False
        )

        if get_cameras.ok:
            cameras = json.loads(get_cameras.text)
            self.cameras = cameras['data']

    def _get_snapshot(self, camera_id):
        """
        Gets a snapshot for the specified camera from the NVR.

        :param camera_id: NVR Camera ID.
        :type camera_id: str
        :returns: `requests` Object
        """
        if not self.cameras:
            self.get_cameras()

        for camera in self.cameras:
            if camera['state'] == 'CONNECTED' and camera['_id'] == camera_id:
                get_snapshot = requests.get(
                    "%s/api/2.0/snapshot/camera/%s" % (
                        self.nvr_url, camera_id),
                    params={'force': 'true', 'apiKey': self.api_key},
                    verify=False
                )

                if get_snapshot.ok:
                    return get_snapshot

    def get_snapshot(self, camera_id):
        """
        Gets a snapshot for the specified camera ID.

        :param camera_id: NVR Camera ID.
        :type camera_id: str
        :returns: Snapshot data (raw).
        :rtype: str
        """
        snapshot = self._get_snapshot(camera_id)
        if snapshot:
            return snapshot.content

    def get_snapshot_url(self, camera_id):
        """
        Gets a snapshot URL for the specified camera ID.

        :param camera_id: NVR Camera ID.
        :type camera_id: str
        :returns: Snapshot URL.
        :rtype: str
        """
        snapshot = self._get_snapshot(camera_id)
        if snapshot:
            return snapshot.url

    def get_all_snapshots(self):
        """
        Gets a snapshot for all cameras from the NVR.

        :returns: All snapshots.
        """
        snapshots = {}

        if not self.cameras:
            self.get_cameras()

        for camera in self.cameras:
            snapshots[camera['_id']] = self.get_snapshot(camera['_id'])
        return snapshots



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""UVSnap Module Constants."""

import logging

__author__ = 'Greg Albrecht <oss@undef.net>'
__copyright__ = 'Copyright 2017 Greg Albrecht'
__license__ = 'Apache License, Version 2.0'


LOG_LEVEL = logging.INFO
LOG_FORMAT = logging.Formatter(
    ('%(asctime)s uvsnap %(levelname)s %(name)s.%(funcName)s:%(lineno)d - '
     '%(message)s'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""UVSnap Commands."""

import argparse
import os
import tempfile

import uvsnap

__author__ = 'Greg Albrecht <oss@undef.net>'
__copyright__ = 'Copyright 2017 Greg Albrecht'
__license__ = 'Apache License, Version 2.0'


def cli():
    """Command Line interface for UVSnap."""

    parser = argparse.ArgumentParser()

    parser.add_argument(
        '-n', '--nvr_url', help='NVR URL'
    )
    parser.add_argument(
        '-a', '--api_key', help='NVR API Key'
    )
    parser.add_argument(
        '-d', '--directory', help='Snapshot Directory',
        default=tempfile.mkdtemp()
    )
    parser.add_argument(
        '-V', '--verbose', help='Verbose Output', action='store_true'
    )
    parser.add_argument(
        'command', metavar='CMD', type=str,
        help='One of: "all", "list" or CAMERA_ID'
    )

    opts = parser.parse_args()

    nvr = uvsnap.NVR(opts.nvr_url, opts.api_key)
    nvr.get_cameras()

    if opts.command == 'list':
        cam_list = []
        for camera in nvr.cameras:
            state = 'UNKNOWN'
            if 'DISCONNECTED' in camera['state']:
                state = 'OFFLINE'
            elif 'CONNECTED' in camera['state']:
                state = 'online'
            _msg = '{:<26}||{:^10}|| {:<30}'.format(
                camera['_id'], state, camera['name'])
            cam_list.append(_msg)

        print("\n".join(cam_list))

    elif opts.command == 'all':
        if not os.path.exists(opts.directory):
            os.makedirs(opts.directory)

        snapshots = nvr.get_all_snapshots()

        for camera_id, snapshot in snapshots.items():
            if snapshot is not None:
                output_file = os.path.join(
                    opts.directory, '%s.jpg' % camera_id)

                with open(output_file, 'wb') as ofd:
                    ofd.write(snapshot)

                if opts.verbose:
                    print('Wrote Snapshot: {}'.format(output_file))

    elif opts.command == 'get_snapshot_url':
        if not nvr.cameras:
            nvr.get_cameras()

        for camera in nvr.cameras:
            print(nvr.get_snapshot_url(camera))
    else:
        camera_id = None
        for camera in nvr.cameras:
            if opts.command in camera['_id']:
                camera_id = camera['_id']

        if camera_id is None:
            print('Camera {} not found.'.format(opts.command))

        if not os.path.exists(opts.directory):
            os.makedirs(opts.directory)

        snapshot = nvr.get_snapshot(camera_id)

        if snapshot is not None:
            output_file = os.path.join(opts.directory, '%s.jpg' % camera_id)

            with open(output_file, 'w') as ofd:
                ofd.write(snapshot)

            if opts.verbose:
                print('Wrote Snapshot: {}'.format(output_file))
